"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const immutable_1 = require("immutable");
const flowie_1 = require("./flowie");
function buildFlowieFromContainer(flowieContainer, flow) {
    if (flow.split) {
        return buildSplitFlow(flow.split, flowieContainer);
    }
    const [firstFlowItemName, ...otherFlowItemNames] = flow.pipe;
    const firstFlowItem = flowieContainer.functionsContainer[firstFlowItemName];
    return buildPipeFlow(immutable_1.Seq.Indexed(otherFlowItemNames), flowieContainer, flowie_1.default(firstFlowItem));
}
exports.default = buildFlowieFromContainer;
function buildSplitFlow(splitFunctions, flowieContainer) {
    const result = splitFunctions.map(getFunctionsFromContainer, flowieContainer);
    return flowie_1.default(...result);
}
function buildPipeFlow(flowItemsNameSequence, flowieContainer, flowieFlow) {
    const flowieItemName = flowItemsNameSequence.first();
    const nextFunctionToPipe = flowieContainer.functionsContainer[flowieItemName];
    const nextStepFlowieFlow = flowieFlow.pipe(nextFunctionToPipe);
    const nextFlowieItemsNameSequence = flowItemsNameSequence.rest();
    if (!nextFlowieItemsNameSequence.size) {
        return nextStepFlowieFlow;
    }
    return buildPipeFlow(nextFlowieItemsNameSequence, flowieContainer, nextStepFlowieFlow);
}
function getFunctionsFromContainer(functionName) {
    return this.functionsContainer[functionName];
}
//# sourceMappingURL=buildFlowieFromContainer.js.map